@@ -4,16 +4,11 @@ library(lattice)
 library(latticeExtra)
 library(grid)
 
-read.signal <- function(filename, mnemonics = NULL, ...) {
-  res <- read.delim(filename, ...)
-  res$label <- factor(res$label)    # adds a label attribute
-
-  # Replace labels with mnemonics, if supplied
-  if (length(mnemonics) > 0) {
-    res$label <- relabel.factor(res$label, mnemonics)
-  } else {
-    res$label <- smart.factor.reorder(res$label)
-  }
+read.signal <- function(filename, mnemonics = NULL, comment.char = "#", ...,
+                        colClasses = list(label = "character")) {
+  comment.lines <- read.comment.lines(filename, comment.char = comment.char)
+  res <- read.delim(filename, ..., colClasses = colClasses)
+  res$label <- relevel.mnemonics(factor(res$label), mnemonics)
 
   res
 }
@@ -204,8 +199,8 @@ xyplot.signal <-
   # this computes the sum of the counts, for the "all" label
   if (mode == "histogram" || mode == "ecdf") {
     labelsums <- with(data, aggregate(count, list(trackname, lower_edge), sum))
-    names(labelsums) <- sub("^x$", "count", names(labelsums))
-    labelsums$label <- "all"
+    names(labelsums) <- c("trackname", "lower_edge", "count")
+    labelsums$label <- factor("all")
     data.full <- rbind(data, labelsums)
     data.full$label <- relevel(data.full$label, ref = "all") 
   } else {
@@ -215,14 +210,13 @@ xyplot.signal <-
   num_rows <- length(levels(data.full$label))
   num_cols <- length(levels(data.full$trackname))
   layout <- c(num_rows, num_cols)
-
-  scales <- panel.scales(data.full, layout, log.y = log.y, mode = mode)
-
   # Remove space between panels (where axes were)
   axis.panel <- rep(c(0, 1), c(num_rows - 1, 1))
   # Make strips wider and longer to fit full text
   strips.heights <- rep(c(strip.height, 0), c(1, num_rows - 1))
   strips.widths <- rep(c(strip.width, 0), c(1, num_cols - 1))
+  
+  scales <- panel.scales(data.full, layout, log.y = log.y, mode = mode)
 
   trellis <- xyplot(x = x,
                     data = data.full,
