@@ -1,6 +1,6 @@
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 180 $"
+__version__ = "$Revision: 201 $"
 
 """
 Assorted utility functions and classes common or useful to most of segtools.
@@ -34,7 +34,6 @@ except NameError:
 PKG_R = os.extsep.join([PKG, "R"])
 PKG_RESOURCE = os.extsep.join([PKG, "resources"])
 
-from .bed import read_native as read_bed
 from .gff import read_native as read_gff
 
 EXT_GZ = "gz"
@@ -74,29 +73,6 @@ r_filename = partial(resource_filename, PKG_R)
 template_string = partial(resource_string, PKG_RESOURCE)
 
 
-## Wrapper for segmentation data object
-class Segmentation(object):
-    """
-    chromosomes: a dict
-      key: chromosome name
-      val: segments: numpy.ndarray, (each row is a (start, end, key) struct)
-           sorted by start, end
-    labels: a dict
-      key: int ("label_key")  (a unique id; segment['end'])
-      val: printable (str or int)  (what's in the actual BED file)
-
-    tracks: a list of track names that were used to obtain the segmentation
-    segtool: the tool that was used to obtain this segmentation (e.g. segway)
-    name: the filename of the se
-    """
-
-    def __init__(self, chromosomes, labels, tracks, segtool, name):
-        self.chromosomes = chromosomes
-        self.labels = labels
-        self.tracks = tracks
-        self.segtool = segtool
-        self.name = name
-
 ## Wrapper for gff/gtf feature data
 class Features(object):
     """
@@ -351,17 +327,12 @@ def image_saver(dirpath, basename, clobber=False, verbose=True,
     Generator to save an R plot to both png and pdf with only one plot
     Yields to caller to plot, then saves images
     """
-    # Make sure there is no silent overwrite
-    filenames = list(make_filename(dirpath, basename, ext)
-                     for ext in IMG_EXTS)
-    for filename in filenames:
-        check_clobber(filename, clobber)
-
     if verbose:
         print >>sys.stderr, "Creating images...",
 
+    png_filename = make_pngfilename(dirpath, basename)
+    check_clobber(png_filename, clobber)
     try:
-        png_filename = make_pngfilename(dirpath, basename)
         r.png(png_filename, **kwargs)
         png_device = r["dev.cur"]()
         r["dev.control"]("enable")
@@ -375,7 +346,7 @@ def image_saver(dirpath, basename, clobber=False, verbose=True,
     # Use R routine to create all the other images (pdf, slide, etc)
     try:
         r["dev.print.images"](basename=basename, dirname=dirpath,
-                              downsample=downsample, **kwargs)
+                              downsample=downsample, clobber=clobber, **kwargs)
         r["dev.off"](png_device)
     except rinterface.RRuntimeError:
         print >> sys.stderr, 'ERROR: Images might not have been saved!'
@@ -383,23 +354,6 @@ def image_saver(dirpath, basename, clobber=False, verbose=True,
     if verbose:
         print >>sys.stderr, "done"
 
-def get_bed_metadata(bedfilename):
-    with maybe_gzip_open(bedfilename) as ifp:
-        return parse_bed_header(ifp.readline())
-
-## Gets track/tool info from BED file header if it exists
-def parse_bed_header(line):
-    regexp = re.compile('description="(.*) segmentation of (.*)"')
-
-    segtool = "Missing from BED file"
-    tracks = ["Missing from BED file"]
-
-    matching = regexp.search(line)
-    if matching:
-        segtool = matching.group(1)
-        tracks = matching.group(2).split(', ')
-    return (segtool, tracks)
-
 
 ## Maps label_keys over segments
 def map_segments(segments, labels, chrom_size):
@@ -584,7 +538,7 @@ def get_ordered_labels(labels, mnemonics=[]):
 ## Returns a numpy.array of strings with a row of [oldlabel, newlabel] for
 ## every old_label, and their ordering specifies the desired display order
 ## Labels should be a dict of label_keys -> label strings
-def map_mnemonics(labels, mnemonicfilename, field="mnemonic"):
+def map_mnemonics(labels, mnemonicfilename, field="new"):
     if mnemonicfilename is None:
         return []
 
@@ -593,7 +547,11 @@ def map_mnemonics(labels, mnemonicfilename, field="mnemonic"):
     mnemonics = []
     # Add mapping for labels in mnemonic file
     for old_label in label_order:
-        new_label = label_data[old_label][field]
+        try:
+            new_label = label_data[old_label][field]
+        except KeyError:
+            die("Mnemonic file missing expected column: %s" % field)
+
         if old_label in str_labels:
             mnemonics.append([old_label, new_label])
 
@@ -611,18 +569,18 @@ def map_mnemonics(labels, mnemonicfilename, field="mnemonic"):
 def load_mnemonics(filename):
     """
     Input file should have a tab-delimited row for each label, of the form:
-               index    description    mnemonic
-      e.g.     4    initiation (strong)    IS
+               old    new    description
+      e.g.     4    IS   initiation (strong)
     Returns a tuple of (label_order, label_data)
 
-    label_order: an ordered list of string label indices,
+    label_order: an ordered list of old labels,
       corresponding to the preferred display order in plots
 
-    label_data: dict
-      key: a string label index
-      value: a dict with fields including "mnemonic" and "description",
+    label_mnemonics: dict
+      key: a string corresponding to an "old" label
+      value: a dict with fields including "new" and "description",
         where description is a several-word description of the label
-        and mnemonic is a few-character identifier
+        and new is a few-character identifier
     """
     if filename is None:
         return []
@@ -634,7 +592,11 @@ def load_mnemonics(filename):
     with open(filename, "rU") as ifp:
         reader = DictReader(ifp)
         for row in reader:
-            label_index = row["index"]
+            try:
+                label_index = row["old"]
+            except KeyError:
+                die("Mnemonic file missing required column: 'old'")
+
             label_order.append(label_index)
             label_data[label_index] = row
 
@@ -826,98 +788,3 @@ def load_features(filename, verbose=True, *args, **kwargs):
         print >>sys.stderr, "done"
 
     return Features(sequences, features, sources)
-
-def bed2arraydict(filename, verbose=True):
-    """Parses a bedfile and returns a dict of numpy arrays for each chrom.
-
-    Returns labels, chromosomes, where:
-      labels: a dict from label_key -> label string
-      chromosomes: a dict from chrom_name -> segments (numpy array)
-        (segments: structured array with start, end, and key fields)
-
-    If the file is in BED3 format, labels will be None
-    """
-    data = defaultdict(list)  # A dictionary-like object
-    label_dict = {}
-    last_segment_start = {}  # per chromosome
-    unsorted_chroms = set()
-    with maybe_gzip_open(filename) as infile:
-        for datum in read_bed(infile):
-            try:
-                if datum.chromStart < last_segment_start[datum.chrom]:
-                    unsorted_chroms.add(datum.chrom)
-            except KeyError:
-                pass
-
-            try:
-                label = str(datum.name)
-            except AttributeError:
-                # No name column, read as BED3 (no labels)
-                label = ""
-
-            try:  # Lookup label key
-                label_key = label_dict[label]
-            except KeyError:  # Map new label to key
-                label_key = len(label_dict)
-                label_dict[label] = label_key
-
-            segment = (datum.chromStart, datum.chromEnd, label_key)
-            data[datum.chrom].append(segment)
-            last_segment_start[datum.chrom] = datum.chromStart
-
-    # Create reverse dict for return
-    labels = inverse_dict(label_dict)
-
-    # convert lists of tuples to array
-    dtype = [('start', DTYPE_SEGMENT_START),
-             ('end', DTYPE_SEGMENT_END),
-             ('key', DTYPE_SEGMENT_KEY)]
-    chromosomes = dict((chrom, array(segments, dtype=dtype))
-                       for chrom, segments in data.iteritems())
-
-    # Sort segments within each chromosome
-    for chrom, segments in chromosomes.iteritems():
-        if chrom in unsorted_chroms:
-            if verbose:
-                print >>sys.stderr, "Segments were unsorted relative to \
-%s. Sorting..." % chrom,
-            segments.sort(order=['start'])
-            if verbose:
-                print >>sys.stderr, "done"
-
-    return labels, chromosomes
-
-def load_segmentation(filename, verbose=True):
-    """Returns a segmentation object derived from the given BED4+ file
-
-    label_keys are integers corresponding to the order observed.
-    """
-
-    if verbose:
-        print >>sys.stderr, "Loading segmentation...",
-
-    # first get the tracks that were used for this segmentation
-    segtool, tracks = get_bed_metadata(filename)
-
-    labels, chromosomes = bed2arraydict(filename)
-
-    if verbose:
-#         print >>sys.stderr, "\n\tMapped labels to integer keys:"
-#         for key, label in labels.iteritems():
-#             print >>sys.stderr, "\t\t\"%s\" -> %d" % (label, key)
-        print >>sys.stderr, "done."
-
-    # wrap in a Segmentation object
-    return Segmentation(chromosomes, labels, tracks, segtool, filename)
-
-## Returns a Genome object from data in a genomedata directory
-def load_genome(genomedatadir):
-    if genomedatadir is not None and os.path.isdir(genomedatadir):
-        genome = Genome(genomedatadir)
-        if not genome or genome is None:
-            die("Error: Unable to load genome data from directory: %s" % \
-                    genomedatadir)
-        else:
-            return genome
-    else:
-        die("Error: Could not find genome data directory: %s" % genomedatadir)
