@@ -13,20 +13,21 @@ distribution over tracks and labels.
 # Must be the same as the folder containing this script
 MODULE="signal_distribution"
 
-__version__ = "$Revision: 178 $"
+__version__ = "$Revision: 192 $"
 
 
 import os
 import sys
 
 from collections import defaultdict
-#from contextlib import contextmanager
+from genomedata import Genome
 from itertools import repeat
 from functools import partial
 from numpy import array, ceil, floor, fromiter, histogram, isfinite, NAN, nanmax, nanmin, nansum, NINF, PINF
 from rpy2.robjects import r, numpy2ri
 
-from .common import die, image_saver, load_segmentation, load_genome, iter_segments_continuous, make_tabfilename, map_mnemonics, r_source, setup_directory, tab_reader, tab_saver
+from . import Segmentation
+from .common import die, image_saver, iter_segments_continuous, make_tabfilename, map_mnemonics, r_source, setup_directory, tab_reader, tab_saver
 from .html import save_html_div
 from .mnemonics import create_mnemonic_file
 
@@ -767,12 +768,15 @@ def save_stats_plot(dirpath, basename=NAMEBASE_STATS, datafilename=None,
                                      translation_file=translation_file,
                                      gmtk=gmtk))
 
-def save_html(dirpath, nseg_dps=None, ecdf=False, clobber=False):
+def save_html(dirpath, genomedatadir, nseg_dps=None,
+              ecdf=False, clobber=False):
     if ecdf:
         title = "%s (ECDF mode)" % HTML_TITLE
     else:
         title = HTML_TITLE
 
+    title = "%s (%s)" % (title, os.path.basename(genomedatadir))
+
     if nseg_dps is None:
         nseg_dps = "???"
 
@@ -790,11 +794,8 @@ def validate(bedfilename, genomedatadir, dirpath, group_labels=False,
              create_mnemonics=False, chroms=None):
 
     setup_directory(dirpath)
-    genome = load_genome(genomedatadir)
-    segmentation = load_segmentation(bedfilename)
-
-    assert genome is not None
-    assert segmentation is not None
+    genome = Genome(genomedatadir)
+    segmentation = Segmentation(bedfilename)
 
     ntracks = genome.num_tracks_continuous  # All tracks; not just segtracks
     segtracks = segmentation.tracks
@@ -827,6 +828,8 @@ def validate(bedfilename, genomedatadir, dirpath, group_labels=False,
                                               calc_ranges=calc_ranges,
                                               value_range=value_range,
                                               quick=quick, chroms=chroms)
+    else:
+        histogram = None
 
     if not replot:
         histogram.save(dirpath, clobber=clobber)
@@ -853,7 +856,8 @@ def validate(bedfilename, genomedatadir, dirpath, group_labels=False,
             nseg_dps = histogram.metadata["nseg_dps"]
         except KeyError:
             nseg_dps = None
-        save_html(dirpath, nseg_dps=nseg_dps, ecdf=ecdf, clobber=clobber)
+        save_html(dirpath, genomedatadir, nseg_dps=nseg_dps,
+                  ecdf=ecdf, clobber=clobber)
 
 
 def parse_options(args):
